function  sim_evalScans(model);

%%---------------------------------------------------------------
% Evaluates time series of simulated fMRI data
% 
% fields of model
%   ROIs   -   definitions of ROIs
%          ROI.name  ... 's'  sphere
%                        'e'  elipse   
%                        'b'  box
%                        'l'  line  
%                        'm'  mask from binary image 
%          ROI.parA  ... center of ROI [mm] or first point of line,  e.g. [10 10 0] 
%          ROI.parB  ... dimension of ROI [mm] or second point of line,  
%                        e.g. [10 10 10] for box or point, [5] for sphere,
%          ROI.w     ... weights of regressors, 
%                        signal in ROI = w(1)*regressor1 + w(2)*regressor2 + .....              
%          ROI.m     ... magnitude of signal (in percents of mean signal in ROI)
%          ROI.rw    ... 'c' constant magnitude of signal over ROI
%                        'l' signal is linearly attenuated 
%                        'g' signal is attenuated in gaussian shape
%
%   X      -   matrix of regressors, each column is one regressor
%
%   Pattern   -   classical "spm_vol" structure of pattern
%                 if Pattern.fname is "defined space" constant background
%                 is used
%
%   Noise   -   noise definition
%          Drift.coef ... coefficients of polynomial reprezentation of
%                           drift drift = c1*time + c2*time^2 + ..... 
%          Drift.val ... value in percents of mean global signal
%          Drift.consider ... 0 no drift , >0 add drift
%          lfn.coef    ... definition of 1/cut_off_frequency 
%          lfn.val     ... value in percents of mean global signal
%          lfn.consider   ... 0 no nf noise , >0 add nf noise
%          gn.val     ... value in percents of mean global signal
%                         gaussian additive noise (random value for each pixel)
%          gn.consider ... 0 no vf noise , >0 add vf noise
%
%   Features   
%          nscan   ...  number of volumes, must correspond to size of
%                       matrix X
%          RT      ...  interscan interval
%          GI      ...  global intensity
%
%   File
%          prefix  ...  names of created files, form: prefix_0000.img, prefix_0001.img ..... 
%          dir     ...  target directory for saving files
%
%
%%---------------------------------------------------------------



%% definition of constants and variables
mat = model.Pattern.mat;
dim = model.Pattern.dim;
nscan = model.Features.nscan;
xyz = cell(0);
Valxyz = cell(0);
meanxyz = [];


%% load (generate) data matrix of underlay image
if strcmp(model.Pattern.fname,'defined space')
    underlayImg = ones(model.Pattern.dim(1:3));
else
    underlayImg = spm_read_vols(model.Pattern);
end


%%  Global-scaling factor 
if strcmp(model.Pattern.fname,'defined space')
    underlayImgMean = 1;
else
   underlayImgMean = spm_global(model.Pattern);
end
GSf = model.Features.GI/underlayImgMean;

%% drift
DriftVals = sim_trend(model.Noise.Drift.coef,model.Features.nscan,model.Noise.Drift.val);

%% compute points of ROIs 
for i = 1:length(model.ROIs)
    xyz{i} = sim_getROI(mat,dim,model.ROIs(i).ROI);
    if model.ROIs(i).ROI.name == 'l'
        center = (model.ROIs(i).ROI.parA + model.ROIs(i).ROI.parB')/2;
    else
        center = model.ROIs(i).ROI.parA;
    end
    center = inv(mat)*[center 1]';
    center = center(1:3)';
    if isfield(model.ROIs(i).ROI,'rwpar')
        rwpar = model.ROIs(i).ROI.rwpar;
    else
        rwpar = 1;
    end
    Valxyz{i} = ROIWeights(xyz{i},center,model.ROIs(i).ROI.rw,rwpar);
end


%%residual files noise
if model.Noise.residn.consider
    [resCount ScR] = sim_getRes(model.Noise.residn.spmFile,model.File.dir,model.File.prefix);
end

%%AR model
if model.Noise.Ar.consider
    C = spm_Ce(nscan,model.Noise.Ar.val);
    V = model.Noise.Ar.h1*C{1} + model.Noise.Ar.h2*C{2};
    % dewhitening matrix
    [u s] = spm_svd(V);
    s = spdiags(1./sqrt(diag(s)),0,nscan,nscan);
    dW = inv(u*s*u');
    dW = dW.*(abs(dW) > 1e-6);
end


Finter = spm('CreateIntWin','on');
set(Finter,'Name','Initializing output volumes....')

cd(model.File.dir);
%% inicialize output files
%-------------------------

if model.File.saveResidImgs
%----Intialise residual image files
Vresid(1:model.Features.nscan) = deal(struct(...
			'fname',	[],...
			'dim',		[dim(1:3) ,spm_type('float')],...
			'mat',		mat,...
			'pinfo',	[1 0 0]',...
			'descrip',	'residual image file',...
            'n',        [],...
            'private',  []));
for i = 1:model.Features.nscan
	Vresid(i).fname   = [model.File.prefix '_resid_' sprintf('%04d.img',i)];
	spm_unlink(Vresid(i).fname)
	Vresid(i)         = spm_create_vol(Vresid(i));
end
end

if model.File.saveSigImgs
%----Intialise signal image files
Vsig(1:model.Features.nscan) = deal(struct(...
			'fname',	[],...
			'dim',		[dim(1:3) ,spm_type('float')],...
			'mat',		mat,...
			'pinfo',	[1 0 0]',...
			'descrip',	'signal image file',...
            'n',        [],...
            'private',  []));
for i = 1:model.Features.nscan
	Vsig(i).fname   = [model.File.prefix '_sig_' sprintf('%04d.img',i)];
	spm_unlink(Vsig(i).fname)
	Vsig(i)         = spm_create_vol(Vsig(i));
end
end

if model.File.saveNoiseImgs
%----Intialise noise image files
Vnoise(1:model.Features.nscan) = deal(struct(...
			'fname',	[],...
			'dim',		[dim(1:3) ,spm_type('float')],...
			'mat',		mat,...
			'pinfo',	[1 0 0]',...
			'descrip',	'noise image file',...
            'n',        [],...
            'private',  []));
for i = 1:model.Features.nscan
	Vnoise(i).fname   = [model.File.prefix '_noise_' sprintf('%04d.img',i)];
	spm_unlink(Vnoise(i).fname)
	Vnoise(i)         = spm_create_vol(Vnoise(i));
end
end

%----Intialise result image files
Vresult(1:model.Features.nscan) = deal(struct(...
			'fname',	[],...
			'dim',		[dim(1:3)],...
            'dt',       [spm_type('float32') 0],...
			'mat',		mat,...
			'pinfo',	[1 0 0]',...
			'descrip',	'result image file',...
            'n',        [],...
            'private',  []));
for i = 1:model.Features.nscan
	Vresult(i).fname   = [model.File.prefix '_result_' sprintf('%04d.img',i)];
	spm_unlink(Vresult(i).fname)
	Vresult(i)         = spm_create_vol(Vresult(i));
%    Vresult(i) = spm_close_vol(Vresult(i));
end

set(Finter,'Name','Computing over planes ....')
spm_progress_bar('Init',dim(3),'Planes computed','')
% --- COMPUTE OVER PLANES ----
for z = 1:dim(3)
    spm_progress_bar('Set',z)

    sigPlane = zeros(nscan,dim(1),dim(2));
    
    for i = 1:length(model.ROIs)
        ind = find(xyz{i}(:,3) == z);
        if ~isempty(ind)
            y = model.X*model.ROIs(i).ROI.w;
            %if (max(y)-min(y)) ~= 0
            %    y = y/(max(y)-min(y));
            %else
            %    y = zeros(1,length(y));
            %end
            for ii = 1:length(ind)
                sigPlane(:,xyz{i}(ind(ii),1),xyz{i}(ind(ii),2)) = Valxyz{i}(ind(ii))*(underlayImgMean/100*model.ROIs(i).ROI.m*y);
            end
        end
    end

    if model.Noise.gn.consider
    for i = 1:dim(1)
        for j = 1:dim(2)
            gn = (underlayImgMean/100)*model.Noise.gn.val*randn(nscan,1);
            if sum(abs(sigPlane(:,i,j)))
                if sum(abs(gn))
                    Pgn = sum(gn.^2)/nscan;
                    mgn = gn.*sigPlane(:,i,j);
                    Pmgn = sum(mgn.^2)/nscan;
                    rgn = sqrt(model.Noise.gn.w*Pgn/Pmgn);
                    gnPlane(:,i,j) = sqrt(1-model.Noise.gn.w)*gn+rgn*mgn;
                else
                    gnPlane(:,i,j) = gn;
                end
            else
                gnPlane(:,i,j) = gn;
            end
        end
    end
    end
    
    if model.Noise.lfn.consider
    for i = 1:dim(1)
        for j = 1:dim(2)
            lfn = (underlayImgMean/100)*model.Noise.lfn.val*sim_lfNoise(nscan,model.Noise.lfn.cutoff,model.Features.RT)';
            if sum(abs(sigPlane(:,i,j)))
                if sum(abs(lfn))
                    Plfn = sum(lfn.^2)/nscan;
                    mlfn = lfn.*sigPlane(:,i,j);
                    Pmlfn = sum(mlfn.^2)/nscan;
                    rlfn = sqrt(model.Noise.lfn.w*Plfn/Pmlfn);
                    lfnPlane(:,i,j) = sqrt(1-model.Noise.lfn.w)*lfn+rlfn*mlfn;
                else
                    lfnPlane(:,i,j) = lfn; 
                end
            else
                lfnPlane(:,i,j) = lfn; 
            end
        end
    end
    end
   
    
    if model.Noise.Ar.consider
        sigPlane = dW*sigPlane;
        gnPlane = dW*gnPlane;
    end
    sigPlane = reshape(sigPlane,nscan,dim(1),dim(2));

    if model.Noise.gn.consider
        gnPlane = reshape(gnPlane,nscan,dim(1),dim(2));
    end
    
    for i = 1:nscan
        s = reshape(sigPlane(i,:,:),dim(1),dim(2));
        n = zeros(size(s));
        if model.Noise.lfn.consider
            n = n + reshape(lfnPlane(i,:,:),dim(1),dim(2));
        end
        if model.Noise.gn.consider
            n = n + reshape(gnPlane(i,:,:),dim(1),dim(2));
        end
        
        if model.File.saveSigImgs
            spm_write_plane(Vsig(i),GSf*s,z);
        end
        if model.File.saveNoiseImgs
            spm_write_plane(Vnoise(i),GSf*n,z);
        end
        
        resultPlane = reshape(s+underlayImg(:,:,z),dim(1),dim(2));
        if model.Noise.gn.consider
            resultPlane = resultPlane + reshape(gnPlane(i,:,:),dim(1),dim(2));
        end
        if model.Noise.lfn.consider
            resultPlane = resultPlane + reshape(lfnPlane(i,:,:),dim(1),dim(2));
        end
        if model.Noise.Drift.consider
            resultPlane = DriftVals(i)*resultPlane;
        end
        if model.Noise.residn.consider
            resind = mod(i-1,resCount)+1;
            Vres = Vresid(1);
            Vres.fname   = [model.File.prefix '_res_' sprintf('%04d.img',resind)];
            residPlane = spm_slice_vol(Vres,[1 0 0 0
                                             0 1 0 0
                                             0 0 0 z
                                             0 0 0 1],[dim(1) dim(2)],0);
            residPlane = model.Features.GI/100*model.Noise.residn.val*residPlane;
            if model.File.saveResidImgs
                spm_write_plane(Vresid(i),residPlane,z);
            end
            spm_write_plane(Vresult(i),GSf*resultPlane+residPlane,z);
        else
            spm_write_plane(Vresult(i),GSf*resultPlane,z);
        end
    end
end
spm_progress_bar('Clear')
delete(spm_figure('FindWin','Interactive'));



%------------------------
%------------------------
function weights = ROIWeights(xyz,c,w,rwpar)
for i = 1:length(xyz(:,1))
    d(i) = sqrt(sum((xyz(i,:)-c).^2));
end
switch w
    case 'l'
        weights = 1-d*rwpar;
    case 'g'
        weights = exp(-(4*d.^2*log(2))/rwpar^2);
    otherwise
        weights = ones(1,length(xyz(:,1)));
end
weights(find(weights<0)) = 0;