function xyz = sim_getROI(mat,dim,ROI)
% computes voxel coordinates of ROI in image space defined by mat and dim
%
% ROI .. ROI structure
%   .name   's' .. sphere 
%           'e' .. elipse    
%           'b' .. box
%           'l' .. line
%           'm' .. binary mask
%
%   .
% dim .. image space dimension
% mat .. vox -> mm transform matrix of image space


xyz = [];
shape = ROI.name;

center = ROI.parA;
siz = ROI.parB;

if ~isfield(ROI,'fi')
    fi = [0 0 0];
else
    fi = ROI.fi;
end

crot = cos(fi/180*pi);
srot = sin(fi/180*pi);

pos = inv(mat)*[center 1]';
pos = pos(1:3)';      

Rx = [ 1   0        0     0
       0 crot(1)  srot(1) 0  
       0 -srot(1) crot(1) 0
       0   0        0     1];
   
Ry = [ crot(2) 0 srot(2) 0
         0     1   0     0
      -srot(2) 0 crot(2) 0
         0     0   0     1];

Rz = [ crot(3)  srot(3) 0 0  
      -srot(3)  crot(3) 0 0
         0        0     1 0     
         0        0     0 1];
     
M1 = [1 0 0 -pos(1)
      0 1 0 -pos(2)
      0 0 1 -pos(3)
      0 0 0     1     ];

M2 = [1 0 0 pos(1)
      0 1 0 pos(2)
      0 0 1 pos(3)
      0 0 0    1     ];

A = inv(M2*Rz*Ry*Rx*M1);  

        
switch shape
    
    % evaluating voxels in 'mask file' case
    case {'M', 'm'}
        
        imgVol = spm_vol(ROI.parFileName);
        B = imgVol.mat;
        iB = inv(B);
                
        [cX cY cZ] = ndgrid(1:dim(1),1:dim(2),1:dim(3));
        
        sX = mat(1,1)*cX + mat(1,2)*cY + mat(1,3)*cZ + mat(1,4);
        sY = mat(2,1)*cX + mat(2,2)*cY + mat(2,3)*cZ + mat(2,4);
        sZ = mat(3,1)*cX + mat(3,2)*cY + mat(3,3)*cZ + mat(3,4);
        
        oX = reshape(iB(1,1)*sX + iB(1,2)*sY + iB(1,3)*sZ + iB(1,4),prod(dim(1:3)),1);
        oY = reshape(iB(2,1)*sX + iB(2,2)*sY + iB(2,3)*sZ + iB(2,4),prod(dim(1:3)),1);
        oZ = reshape(iB(3,1)*sX + iB(3,2)*sY + iB(3,3)*sZ + iB(3,4),prod(dim(1:3)),1);
        
        nvolume = spm_sample_vol(imgVol,oX,oY,oZ,1);
        
        [xs ys zs] = ind2sub(dim,find(nvolume));
        xyz = [xs ys zs];
        
        
    
    % evaluating voxels in 'box' case
    case {'B', 'b'}
        
        [cX cY cZ] = ndgrid(1:dim(1),1:dim(2),1:dim(3));
        sX = round(A(1,1)*cX + A(1,2)*cY + A(1,3)*cZ + A(1,4));
        sY = round(A(2,1)*cX + A(2,2)*cY + A(2,3)*cZ + A(2,4));
        sZ = round(A(3,1)*cX + A(3,2)*cY + A(3,3)*cZ + A(3,4));

        try
            siz = siz(1:3);
            d = [mat(1,1) mat(2,2) mat(3,3)];
            dpx = abs(siz./d);
        catch
            return
        end

        ind = find(abs(sX-pos(1))<dpx(1) & abs(sY-pos(2))<dpx(2) & abs(sZ-pos(3))<dpx(3));
        xyz = [cX(ind) cY(ind) cZ(ind)];
        
        
    % evaluating voxels in 'line' case        
    case {'L', 'l'}
        try
            iA = (inv(mat)*[center(1:3)';1])'; iA = round(iA(1:3));
            iB = (inv(mat)*[siz(1:3);1])'; iB = round(iB(1:3));
        catch
            return
        end

        if (sum(iA<1)==0) & (sum(iA>dim(1:3))==0)
            xyz = [iA];
        end
        
        if iA == iB
            return
        end
       
        iC = iA;
        up = iB-iA;  
        
        while sum(iC==iB) < 3
            us = sign(iB-iC);
            if us(1) == 0
                ux = [0];
            else
                ux = [0 us(1)];
            end
            if us(2) == 0
                uy = [0];
            else
                uy = [0 us(2)];
            end
            if us(3) == 0
                uz = [0];
            else
                uz = [0 us(3)];
            end
            
            e = [];
            for i = ux
                for j = uy
                    for k = uz
                        e = [e;[i j k]];
                    end
                end
            end
            n = length(e(:,1))-1;
            eN = zeros(n,3);
            delta = zeros(n,1);
            for i = 2:n+1
                eN = iC+e(i,:);
                d = -up*eN';
                t = (-d-up*iA')/sum(up.^2);
                P = iA+t*up;
                delta(i-1) = sum((P-eN).^2); 
            end
            
            ind = find(delta == min(delta)); 
            iC = iC + e(ind(1)+1,:);
            
            if (sum(iC<1)==0) & (sum(iC>dim(1:3))==0)
                xyz = [xyz; iC];
            else
                iC = iB;
            end
        end


    % evaluating voxels in 'sphere' or 'elipse' case        
    case {'S', 's', 'e', 'E'}

        [cX cY cZ] = ndgrid(1:dim(1),1:dim(2),1:dim(3));
        sX = round(A(1,1)*cX + A(1,2)*cY + A(1,3)*cZ + A(1,4));
        sY = round(A(2,1)*cX + A(2,2)*cY + A(2,3)*cZ + A(2,4));
        sZ = round(A(3,1)*cX + A(3,2)*cY + A(3,3)*cZ + A(3,4));

        try
            if isempty(siz)
                return
            elseif length(siz) == 1
                siz = [siz siz siz];
            end
            d = [mat(1,1) mat(2,2) mat(3,3)];
            rad = abs(siz./d);
        catch
            return
        end
        
        rad2 = rad.^2;
        
        ind = find(1 > ((pos(1)-sX).^2/rad2(1) + (pos(2)-sY).^2/rad2(2) + (pos(3)-sZ).^2/rad2(3)));
        xyz = [cX(ind) cY(ind) cZ(ind)];
end


  
